/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package coinserver.blackjack;

/**
 *
 * @author Yury V. Reshetov
 */
import paymentsystem.*;
import menu.BlackJackMenu;
import menu.Menu;
import java.util.*;
import java.net.*;
import java.io.*;
import java.math.*;
import coinserver.*;


/* Simulation of console-I/O program Blackjack,
using ConsoleApplet as a basis.  See the file
ConsoleApplet.java for more information.

David Eck
eck@hws.edu

 */
public class BlackjackConsole
{

    private static Scanner scanner = new Scanner(System.in);
    private Card[] deck;   // An array of 52 Cards, representing the deck.
    private int currentPosition; // Current position in the deck
    private ArrayList hand;   // The cards in the hand.
    private java.security.SecureRandom rand = new java.security.SecureRandom();
    private int bet;            // Amount user bets on a game.


    public BlackjackConsole(BufferedReader br, PrintWriter pw)
    {
        /*
        This program lets the user play Blackjack.  The computer
        acts as the dealer.  The user has a stake of $100, and
        makes a bet on each game.  The user can leave at any time,
        or will be kicked out when he loses all the money.
        House rules:  The dealer hits on a total of 16 or less
        and stands on a total of 17 or more.  Dealer wins ties.
        A new deck of cards is used for each game.
         */
        try {
            int money;          // Amount of money the user has.
            int userWins;   // Did the user win the game?
            pw.println();
            pw.println("Welcome to the game of blackjack.");
            pw.println();
            Thread.sleep(1000);

            money = 1;  // User starts with 1.

            while (true)
            {
                pw.println("You have " + money + " of 128 coins");
                bet = money;
                pw.flush();
                userWins = playBlackjack(br, pw);
                if (userWins > 0) {
                    if (userWins == 2) {
                        bet = bet * 3;
                        bet = bet / 2;
                    }
                    money = money + bet;
                } else {
                    if (userWins != 0) {
                        if (userWins == -2) {
                            bet = bet / 2;
                        }
                        money = money - bet;
                    }
                }
                pw.println();
                if (money == 0) {
                    pw.println("Looks like you've are out of money!");
                    break;
                }
                if (money >= 128) {
                    pw.println("Enter the number of new wallet:");
                    pw.flush();
                    BigInteger number = new BigInteger(br.readLine().trim(), 16);
                    pw.println("Your wallet:");
                    pw.println();
                    pw.println("Number   : " + number.toString(16));
                    Wallet wallet = new Wallet(number, new LogsManager());
                    wallet.addCoins(new BigDecimal(money), wallet.getCoinsCount() - 1);
                    break;
                }
            }

            pw.println();
            pw.println("You leave with " + money + '.');
        } catch (Exception ex) {
            //ex.printStackTrace();
        }

    } // end main()

    private int playBlackjack(BufferedReader br, PrintWriter pw) {
        // Let the user play one game of Blackjack.
        // Return true if the user wins, false if the user loses.
        try {

            ArrayList dealerHand;   // The dealer's hand.
            ArrayList userHand;     // The user's hand.

            // Create an unshuffled deck of cards.
            deck = new Card[208];
            int cardCt = 0; // How many cards have been created so far.
                for (int i = 0; i < 4; i++) {
                    for (int suit = 0; suit < 4; suit++) {
                        for (int value = 1; value <= 13; value++) {
                            deck[cardCt] = new Card(value, suit, this);
                            cardCt++;
                    }
                }
            }
            currentPosition = 0;

            dealerHand = new ArrayList();
            userHand = new ArrayList();

            /*  Shuffle the deck, then deal two cards to each player. */

            shuffle();

            dealerHand.add(dealCard());
            dealerHand.add(dealCard());
            userHand.add(dealCard());
            userHand.add(dealCard());


            /* Check if one of the players has Blackjack (two cards totaling to 21).
            The player with Blackjack wins the game.  Dealer wins ties.
            */

            if (value(dealerHand) == 21) {
                Thread.sleep(1000);
                pw.println("Dealer has the " + getCard(dealerHand, 0) + " and the " + getCard(dealerHand, 1) + ".");
                pw.println("User has the " + getCard(userHand, 0) + " and the " + getCard(userHand, 1) + ".");
                pw.println();
                pw.println("Dealer has Blackjack.  Dealer wins.");
                return -1;
            }

            if (value(userHand) == 21) {
                Thread.sleep(1000);
                pw.println("Dealer has the " + getCard(dealerHand, 0) + " and the " + getCard(dealerHand, 1) + ".");
                pw.println("User has the " + getCard(userHand, 0) + " and the " + getCard(userHand, 1) + ".");
                pw.println();
                pw.println("You have Blackjack.  You win.");
                return 2;
            }

                
                
            /*  If neither player has Blackjack, play the game.  The user gets a chance
            to draw cards (i.e., to "Hit").  The while loop ends when the user
            chooses to "Stand" or when the user goes over 21.
            */

            while (true) {

                /* Display user's cards, and let user decide to Hit or Stand. */

                pw.println();
                pw.println("Your cards are:");
                for (int i = 0; i < userHand.size(); i++) {
                    pw.println("    " + getCard(userHand, i));
                }
                pw.println("Your total is " + value(userHand));
                pw.println();
                pw.println("Dealer is showing the " + getCard(dealerHand, 0));
                Menu menu = new BlackJackMenu();
                menu.addPoint("1. Hit");
                menu.setPoint(1);
                if (this.isSurrender(userHand)) {
                    menu.addPoint("2. Surrender");
                    menu.setPoint(2);
                }
                char useraction = '0';  // User's response, 'H' or 'S'.
                String command = "";
                do {
                    pw.println(menu);
                    pw.flush();
                    command = br.readLine().trim();
                } while (! menu.checkCommand(command));
                Thread.sleep(1000);
                useraction = command.charAt(0);
                /* If the user Hits, the user gets a card.  If the user Stands, the
                dealer gets a chance to draw and the game ends.
                */
                
                if ((useraction == '2') && this.isSurrender(userHand)) {
                    return -2;
                }

                if (useraction == '0') {
                    // Loop ends; user is done taking cards.
                    break;
                } else {  // useraction is 'H'.
                    // Give the user a card.  If the user goes over 21, the user loses.
                    Card newCard = dealCard();
                    userHand.add(newCard);
                    pw.println();
                    pw.println("User hits.");
                    pw.println("Your card is the " + newCard);
                    pw.println("Your total is now " + value(userHand));
                    if (value(userHand) > 21) {
                        pw.println();
                        pw.println("You busted by going over 21.  You lose.");
                        pw.println("Dealer's other card was the " + getCard(dealerHand, 1));
                        return -1;
                    }
                }

            } // end while loop

            /* If we get to this point, the user has Stood with 21 or less.  Now, it's
            the dealer's chance to draw.  Dealer draws cards until the dealer's total is > 16.
            */

            pw.println();
            pw.println("User stands.");
            pw.println("Dealer's cards are:");
            pw.println("    " + getCard(dealerHand, 0));
            pw.println("    " + getCard(dealerHand, 1));
            while (value(dealerHand) < 17) {
                Card newCard = dealCard();
                pw.println("Dealer hits and gets the " + newCard);
                dealerHand.add(newCard);
            }
            pw.println("Dealer's total is " + value(dealerHand));

            /* Now, the winner can be declared. */

            pw.println();
            if (value(dealerHand) > 21) {
                pw.println("Dealer busted by going over 21.  You win.");
                return 1;
            } else {
                if (value(dealerHand) == value(userHand))
                {
                    pw.println("Push.");
                    return 0;
                } else {
                    if (value(dealerHand) > value(userHand)) {
                        pw.println("Dealer wins, " + value(dealerHand) + " points to " + value(userHand) + ".");
                        return -1;
                    } else {
                        pw.println("You win, " + value(userHand) + " points to " + value(dealerHand) + ".");
                        return 1;
                    }
                }
            }
         } catch (Exception ex) {
            ex.printStackTrace();
        }
        return 0;
    }  // end playBlackjack()

    public Card dealCard()
    {
        // Deals one card from the deck and returns it.
        currentPosition++;
        if (currentPosition == 104)
        {
            shuffle();
        }
        return deck[currentPosition - 1];
    }

    public void shuffle()
    {
        // Put all the used cards back into the deck, and shuffle it into
        // a random order.
        for (int i = 207; i > 0; i--)
        {
            int rnd = (int) (Math.random() * (i + 1));
            Card temp = deck[i];
            deck[i] = deck[rnd];
            deck[rnd] = temp;
        }
        currentPosition = 0;
    }

    public Card getCard(ArrayList hand, int position)
    {
        // Get the card from the hand in given position, where positions
        // are numbered starting from 0.  If the specified position is
        // not the position number of a card in the hand, then null
        // is returned.
        if (position >= 0 && position < hand.size())
        {
            return (Card) hand.get(position);
        } else
        {
            return null;
        }
    }

    public int value(ArrayList hand)
    {
        // Returns the value of this hand for the
        // game of Blackjack.

        int val;      // The value computed for the hand.
        int ace = 0;  // This will be set to true if the
        //   hand contains an ace.
        int cards;    // Number of cards in the hand.

        val = 0;
        cards = hand.size();

        for (int i = 0; i < cards; i++)
        {
            // Add the value of the i-th card in the hand.
            Card card;    // The i-th card;
            int cardVal;  // The blackjack value of the i-th card.
            card = getCard(hand, i);
            cardVal = card.getValue();  // The normal value, 1 to 13.
            if (cardVal > 10)
            {
                cardVal = 10;   // For a Jack, Queen, or King.
            }
            if (cardVal == 1)
            {
                ace++;     // There is at least one ace.
                val = val + 10;
            }
            val = val + cardVal;
            if ((val > 21) && (ace > 0)) {
                val = val - 10;
                ace--;
            }
        }

        // Now, val is the value of the hand, counting any ace as 1.
        // If there is an ace, and if changing its value from 1 to
        // 11 would leave the score less than or equal to 21,
        // then do so by adding the extra 10 points to val.


        return val;

    }
    
    private BigInteger get128bitsNumber() {
        BigInteger result = new BigInteger(128, this.rand);
        while (result.bitLength() != 128) {
            result = new BigInteger(128, this.rand);
        }
        return result;
    }
    
    private boolean isSurrender(ArrayList hand) {
        return ((this.bet > 0) && (hand.size() == 2));
    }
    
}  // end class