/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package paymentsystem;

import java.math.*;
import java.io.*;

/**
 *
 * @author Yury V. Reshetov
 */

public class Market {
    private BigDecimal[] coins = null;
    private BigDecimal two = new BigDecimal("2");
    private LogsManager lm = null;
    
    public Market(LogsManager logsmanager) {
        this.lm = logsmanager;
        this.load();
    }
    
    private void load() {
        try {
            int coinslength = new Wallet(BigInteger.ZERO, this.lm).getCoinsCount();
            this.coins = new BigDecimal[coinslength];
            File f = new File("market.dat");
            if (f.exists()) {
                BufferedReader br = new BufferedReader(new FileReader(f));
                for (int i = 0; i < this.coins.length; i++) {
                    String s = br.readLine();
                    this.coins[i] = new BigDecimal(s);
                }
                br.close();
            } else {
                BigDecimal begin = new BigDecimal("10000000");
                for (int i = this.coins.length - 1; i >= 0; i--) {
                    this.coins[i] = begin;
                    begin = begin.divide(BigDecimal.TEN, 0, BigDecimal.ROUND_CEILING);
                }
                this.coins[this.coins.length - 1] = this.coins[this.coins.length - 2];
                this.save();
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    public BigDecimal getPrice(int firstcoin, int secondcoin, BigDecimal volume) {
        return this.coins[firstcoin].divide(this.coins[secondcoin].add(volume.multiply(this.two)), 12, BigDecimal.ROUND_DOWN);
    }
    

    public synchronized BigDecimal deal(int firstcoin, int secondcoin, BigDecimal volume, Wallet wallet, BigInteger pin) {
        BigDecimal firstvolume = this.getPrice(firstcoin, secondcoin, volume).multiply(volume).divide(BigDecimal.ONE, 6, BigDecimal.ROUND_DOWN);
        synchronized (wallet) {
            if (wallet.getCoins(secondcoin).compareTo(volume) >= 0) {
                this.coins[firstcoin] = this.coins[firstcoin].subtract(firstvolume);
                wallet.addCoins(firstvolume, firstcoin);
                wallet.subtractCoins(volume, secondcoin, BigInteger.ZERO);
                this.coins[secondcoin] = this.coins[secondcoin].add(volume);
                this.save();
                this.lm.setLog(BigInteger.ZERO, wallet.getNumber(), firstcoin, firstvolume);
            } else {
                firstvolume = BigDecimal.ZERO;
            }
        }
        return firstvolume;
    }

    public BigDecimal getReversePrice(int firstcoin, int secondcoin, BigDecimal volume) {
        return this.coins[secondcoin].divide(this.coins[firstcoin].subtract(volume.multiply(this.two)), 12, BigDecimal.ROUND_DOWN);
    }

    public synchronized BigDecimal reverseDeal(int firstcoin, int secondcoin, BigDecimal volume, Wallet wallet, BigInteger pin) {
        BigDecimal firstvolume = this.getReversePrice(firstcoin, secondcoin, volume).multiply(volume).divide(BigDecimal.ONE, 6, BigDecimal.ROUND_UP);
        synchronized (wallet) {
            if (wallet.getCoins(secondcoin).compareTo(firstvolume) >= 0) {
                this.coins[firstcoin] = this.coins[firstcoin].subtract(volume);
                wallet.addCoins(volume, firstcoin);
                wallet.subtractCoins(firstvolume, secondcoin, BigInteger.ZERO);
                this.coins[secondcoin] = this.coins[secondcoin].add(firstvolume);
                this.save();
                this.lm.setLog(BigInteger.ZERO, wallet.getNumber(), firstcoin, volume);
            } else {
                firstvolume = BigDecimal.ZERO;
            }
        }
        return firstvolume;
    }

    private void save() {
        try {
            File f = new File("market.dat");
            PrintWriter pw = new PrintWriter(new FileWriter(f));
            for (int i = 0; i < this.coins.length; i++) {
                pw.println(coins[i]);
            }
            pw.flush();
            pw.close();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
}

